<?php

namespace Drupal\mapbox_field\Plugin\views\style;

use Drupal\core\form\FormStateInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactory;

/**
 * Allow to display several field items on a common map.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "single_map",
 *   title = @Translation("Mapbox - All in one"),
 *   help = @Translation("Display all markers on one map."),
 *   theme = "views_view_single_map",
 *   display_types = {"normal"},
 * )
 */
class SingleMap extends StylePluginBase {

  /**
   * Does the style plugin for itself support to add fields to its output.
   *
   * @var bool
   */
  protected $usesFields = TRUE;

  /**
   * Does the style plugin support grouping of rows.
   *
   * @var bool
   */
  protected $usesGrouping = FALSE;

  protected $accessToken;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactory $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->accessToken = $config_factory->get('mapbox_field.mapboxconfig')->get('access_token');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['threeDBuildings'] = ['default' => 1];
    $options['keyboard']        = ['default' => 0];
    $options['navigation']      = ['default' => 0];
    return $options;
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $fields = $this->displayHandler->getOption('fields');
    $labels = $this->displayHandler->getFieldLabels();
    $map_fields_options = [];
    foreach ($fields as $field_name => $field) {
      if ($field['type'] == 'mapbox_raw_formatter') {
        $map_fields_options[$field_name] = $labels[$field_name];
      }
    }
    $form['mapbox_settings'] = [
      '#type' => 'fieldset',
      '#title' => t('Mapbox Settings'),
      '#open' => TRUE
    ];
    $form['source_field'] = [
      '#type' => 'select',
      '#title' => t('Mapbox Source Field'),
      '#fieldset' => 'mapbox_settings',
      '#required' => TRUE,
      '#options' => $map_fields_options,
      '#default_value' => (isset($this->options['source_field'])) ? $this->options['source_field'] : ''
    ];
    $form['threeDBuildings'] = [
      '#type' => 'checkbox',
      '#title' => t('3D Buildings'),
      '#fieldset' => 'mapbox_settings',
      '#default_value' => (isset($this->options['threeDBuildings'])) ? $this->options['threeDBuildings'] : 0,
    ];
    $form['keyboard'] = [
      '#type' => 'checkbox',
      '#title' => t('Keyboard Controls'),
      '#fieldset' => 'mapbox_settings',
      '#default_value' => (isset($this->options['keyboard'])) ? $this->options['keyboard'] : 0,
    ];
    $form['navigation'] = [
      '#type' => 'checkbox',
      '#title' => t('Navigation Controls'),
      '#fieldset' => 'mapbox_settings',
      '#default_value' => (isset($this->options['navigation'])) ? $this->options['navigation'] : 0,
    ];
  }

  public function render() {
    $source_field = $this->options['source_field'];
    foreach ($this->view->result as $row) {
      $mapbox_field = $this->view->field[$source_field];
      $mapbox_field_items = $mapbox_field->getItems($row);
      foreach ($mapbox_field_items as $delta => $item) {
        $mapbox = $item['raw'];
        $positions[] = [
          'lat' => (float) $mapbox->lat,
          'lng' => (float) $mapbox->lng,
          'marker' => $mapbox->marker ? $mapbox->marker : '/'.drupal_get_path('module', 'mapbox_field').'/marker.png',
          'markerText' => $mapbox->markerText ?? $mapbox->markerText,
        ];
      }
    }

    $mapbox_settings['accessToken'] = $this->accessToken;
    $mapbox_settings['options']     = $this->options;
    $mapbox_settings['positions']   = $positions;
    $mapbox_settings['center']      = $positions[0];

    $build = [
      '#theme'  => 'mapbox_field_singlemap',
      '#mapbox' => $mapbox_settings['positions']
    ];

    $build['#attached']['library'][] = 'mapbox_field/mapbox-singlemap';
    $build['#attached']['library'][] = 'mapbox_field/mapbox-apis';
    $build['#attached']['drupalSettings']['mapbox_singlemap'] = $mapbox_settings;

    return $build;
  }
}
